// ETH044 Java example

import java.net.*;
import java.io.*;
import java.util.*;

public class ETH044
{
	static ETH044.NetworkSocket network;
	static Scanner user_input = new Scanner(System.in);
	
	public static void main(String args[])
	{
		network = new ETH044.NetworkSocket(args[0], Integer.parseInt(args[1]));
		if (network.connected)
		{
			if (check_password() == -1) return;
			input_loop();
		}
		network.closeStreams();
	}
	
	private static int check_password()
	{
		network.write(new byte[]{0x7A});						// Check lock time byte to see if password is enabled
		byte[] data = network.read(1); 
		
		if (data[0] == 0)
		{
			System.out.print("Enter password: ");				// Prompt user to input the password if it is enabled
			String password = user_input.nextLine();
			network.write("y".concat(password).getBytes());		// Put the password command together, the "y" here is the byte needed to send the password (0x79 is y in ascii)
			data = network.read(1);
			
			if (data[0] != 1)									// Check to see if password was excepted
			{
				System.out.println("Wrong password");
				return -1;
			}
		}
		
		return 1;
	}
	
	private static void output_states()
	{
		network.write(new byte[]{0x24});						// Command to get the output states
		byte[] data = network.read(2); 
		System.out.println("SSR states: " + String.format("%4s", Integer.toBinaryString((data[0] & 0xFF))).replace(' ', '0'));
		System.out.println("Output states: " + String.format("%4s", Integer.toBinaryString((data[1] & 0xFF))).replace(' ', '0'));
		
		network.write(new byte[]{0x25});						// Command to get the input states
		data = network.read(2); 
		System.out.println("Input states: " + String.format("%4s", Integer.toBinaryString((data[1] & 0xFF))).replace(' ', '0'));
	}
	
	private static void input_loop()
	{
		System.out.println("\nTo set an SSR to a value enter a1-a4 for which SSR you wish to change followed by 1-100 for the % power to set it to \nEnter number of an output followed by on or off to set that output e.g. \"1 on\". \nSetting an SSR to \"on\" will set it to full power, setting it to \"off\" will set it to minimum power. \nTo view states just enter \"states\".\nTo exit enter \"exit\" ");
		String input = null;
		 while (true)
		 {
			System.out.print(">> ");				// Get the command from  the user
			input = user_input.nextLine();
			if (input.equals("exit"))				// Deal with command
			{
				System.exit(0);
			}
			else if (input.contains("on"))
			{
				output_toggle(input, 1);
			}
			else if (input.contains("off"))
			{
				output_toggle(input, 0);
			}
			else if (input.contains("a1") || input.contains("a2") || input.contains("a3") || input.contains("a4"))
			{
				ssr_set(input);
			}
			else if (input.equals("states"))
			{
				output_states();
			}
			else {
				System.out.println("\nTo set an SSR to a value enter a1-a4 for which SSR you wish to change followed by 1-100 for the % power to set it to \nEnter number of an output followed by on or off to set that output e.g. \"1 on\". \nSetting an SSR to \"on\" will set it to full power, setting it to \"off\" will set it to minimum power. \nTo view states just enter \"states\".\nTo exit enter \"exit\" ");
			}
		 }
		
	}
	
	private static void ssr_set(String input)
	{
		String[] parts = input.split(" ");
		byte output = 0;
		
		try{
			output = (byte)Integer.parseInt(parts[1]);							// Get the value we are setting the SSR to
		}catch(NumberFormatException e){
			
			System.out.println("Output value not real number." + parts[0]);
			return;
		}
		
		if (output > 100)output = 100;						
		
		byte ssr = 0;
		if (parts[0].equals("a1")) {
			network.write(new byte[]{0x30, 0x01, 0x00, output, 0x00});
		}
		else if (parts[0].equals("a2")) {
			network.write(new byte[]{0x30, 0x02, 0x00, output, 0x00});
		}
		else if (parts[0].equals("a3")) {
			network.write(new byte[]{0x30, 0x03, 0x00, output, 0x00});
		}
		else if (parts[0].equals("a4")) {
			network.write(new byte[]{0x30, 0x04, 0x00, output, 0x00});
		}
	}
	
	private static void output_toggle(String input, int state)
	{
		String[] parts = input.split(" ");										// Split the input command by whitespace
		byte output = 0;
		
		try{
			output = (byte)Integer.parseInt(parts[0]);							// First part of command is the output number, aprse this as a byte
		}catch(NumberFormatException e){
			
			System.out.println("Output value not real number." + parts[0]);
			return;
		}

		if ((output > 0 && output <= 4) || (output > 8 && output <= 16))
		{
			if (state == 1)
			{
				network.write(new byte[]{0x20, output, 0x00});					// Set output active
			}
			else
			{
				network.write(new byte[]{0x21, output, 0x00});					// Set output inactive
			}	
			network.read(1);
		}
	}
	
	/* 
	Embeded class that deals with all network traffic -------------------------------------------------------------------
	*/
	static class NetworkSocket
	{
		private Socket nw;
		private InputStream i_stream;
		private OutputStream o_stream;
		
		public boolean connected = false;
		
		public NetworkSocket()
		{
			
		}
		
		public NetworkSocket(String ip_address, int port)
		{
			try
			{
				nw = new Socket(ip_address, port);
				i_stream = nw.getInputStream();
				o_stream = nw.getOutputStream();
				connected = true;
			} 
			catch (IOException e)
			{
				System.err.println("Caught IOException: " + e.getMessage());
				return;
			}

		}
		
		public void closeStreams()
		{
			write(new byte[]{0x7B});
			
			try
			{
				i_stream.close();
				o_stream.close();
				nw.close();
			}
			catch (IOException e)
			{
				System.err.println("Caught IOException: " + e.getMessage());
				return;
			}
		}
		
		public void write(byte msg[])
		{
			int length = msg.length;
			try
			{
				o_stream.write(msg);
			}
			catch (IOException e)
			{
				System.err.println("Caught IOException: " + e.getMessage());
				return;
			}
		}
		
		public byte[] read(int count)
		{
			byte[] data = new byte[32];
			int off = 0;
			int num = 0;
			
			try
			{
				num = i_stream.read(data,off,count);
			}
			catch (IOException e)
			{
				System.err.println("Caught IOException: " + e.getMessage());
				return null;
			}
			catch (NullPointerException e)
			{
				System.err.println("Caught NullPointerExceptionn: " + e.getMessage());
				return null;
			}
			catch (IndexOutOfBoundsException e)
			{
				System.err.println("Caught IndexOutOfBoundsException: " + e.getMessage());
				return null;
			}
			
			return data;
		}
	}
}
